// csPEformat.cs

	// coff 形式( PE 形式) の exe/dll ファイルのrawデータを
	// 入出力する。


	// 当初は VB.net のみで実装予定だったが、VBfixedArray

	// https://learn.microsoft.com/ja-jp/dotnet/api/microsoft.visualbasic.vbfixedarrayattribute?view=net-8.0
	//		VBFixedArrayAttribute クラス
	//			Structure Book
	//				<VBFixedArray(4)> Public Chapter() As Integer
	//			End Structure
	
	// こんな感じで、固定配列をVBでも一応は使えるが、
	// ・中身は VB6 32bit コードではないか？
	// ・将来的に同機能はサポートが打ち切られるのではないのか？
	// ・ちょっと面倒な動きをさせただけで、全く動かないのでは？
	//等々を考慮した結果、
	// ・Csharp の_unsafe struct ... Fixed  宣言を使用。
	// ・上記を CSharpコード上で、マネージド構造体にメンバごと転送。
	//という方式を取る事にした。



using System;





//#if AAA
//#endif


using DWORD = System.UInt32;
using DWORD64 = System.UInt64;

using WORD = System.UInt16;


using BOOL = System.Int32;

// csharp/vb/win32 vc/vc compiler
//  各種データ型のサイズ詳細メモは csWin32.cs を参照。





namespace CliDbger {



internal partial class clsPeFormat {


	//	https://learn.microsoft.com/ja-jp/windows/win32/debug/pe-format
	//			PE 形式
	//	windows-win32-debug.pdf
	//			1187 ページ
	//				PE 形式



	/*****************************************************/
	/*      PE ファイル、データ実体                      */
	/*****************************************************/


	internal unsafe struct stPE {
			internal stMSDOS_Header				msdos;
			internal stPeFileHeader				pefile;

			//internal stOptionalHeader_32		optional;
			//または
			//internal stOptionalHeader_64		optional;


			// stSectionHeader   sec[ secnumber ]

	}



	/*****************************************************/
	/*      各構造体                                     */
	/*****************************************************/

	internal unsafe struct stMSDOS_Header {
		internal fixed byte		hedaderchar[ 0x80 ];		//  C sharp だと、要素数＝宣言時の添え字の数なんだってさー。
		internal fixed byte		PESignature[ 4 ];
	}

	private static byte[] mbytMSDOS_HeaderChar = { 
		0x4d, 0x5a, 0x90, 0x0 , 0x03, 0x0,  0x0,  0x0,    0x04, 0x0,  0x0,  0x0,  0xff, 0xff, 0x0,  0x0,   
		0xb8, 0x0,  0x0,  0x0,  0x0,  0x0,  0x0,  0x0,    0x40, 0x0,  0x0,  0x0,  0x0,  0x0,  0x0,  0x0,  
		0x0,  0x0,  0x0,  0x0,  0x0,  0x0,  0x0,  0x0,    0x0,  0x0,  0x0,  0x0,  0x0,  0x0,  0x0,  0x0,  
		0x0,  0x0,  0x0,  0x0,  0x0,  0x0,  0x0,  0x0,    0x0,  0x0,  0x0,  0x0,  0x80, 0x0,  0x0,  0x0,  
		                                                    
		0x0e, 0x1f, 0xba, 0xe,  0x0,  0xb4, 0x09, 0xcd,   0x21, 0xb8, 0x01, 0x4c, 0xcd, 0x21, 0x54, 0x68, 
		0x69, 0x73, 0x20, 0x70, 0x72, 0x6f, 0x67, 0x72,   0x61, 0x6d, 0x20, 0x63, 0x61, 0x6e, 0x6e, 0x6f, 
		0x74, 0x20, 0x62, 0x65, 0x20, 0x72, 0x75, 0x6e,   0x20, 0x69, 0x6e, 0x20, 0x44, 0x4f, 0x53, 0x20, 
		0x6d, 0x6f, 0x64, 0x65, 0x2e, 0xd,  0xd,  0xa,    0x24, 0x0,  0x0,  0x0,  0x0,  0x0,  0x0,  0x0  
	};
	private static byte[] mbytMSDOS_PESignature = { 0x50, 0x45, 0x0, 0x0 };		// "" PE\0\0 ""


	internal struct stPeFileHeader {
		

		// offset 0
		internal ushort				Machine;			// enum colonyPEFILE_MACHINE.IMAGE_FILE_MACHINE_I386
		// offset 2
		internal ushort				NumberOfSections;

		// offset 4
		// ファイルが作成された日時を示す、1970 年 1 月 1 日 00:00 以降の秒数の下位 32 ビット (C ランタイム time_t値)。
		// こんなん作れるんかいな・・・。
		internal uint				TimeDateStamp;

		// offset 8
		internal uint				PointerToSymbolTable;
		// offset 12
		internal uint				 NumberOfSymbols;
		// offset 16
		internal ushort				SizeOfOptionalHeader;
		// offset 18
		internal ushort				Characteristics;		// enum colonyPEFILE_CHARACTERISTICS
	}		// struct stPeFileHeader



	// 中身はこんなだが、実際には使わない。warning 勝手に出てしまうので
	// とりあえずコメントアウト
	#pragma warning disable 169
	internal unsafe struct stOptionalHeader_32 
	{
		stStanderdCoffFields				StanderdCoff;
		stWindowsNT_specificFields_32		WindowsSpecific;
		stDataDirectories					DataDirectories;
	}		// struct stOptionalHeader

	internal unsafe struct stOptionalHeader_64 
	{
		stStanderdCoffFields				StanderdCoff;
		stWindowsNT_specificFields_64		WindowsSpecific;
		stDataDirectories					DataDirectories;
	}		// struct stOptionalHeader
	#pragma warning restore  169



	internal struct stStanderdCoffFields
	{
		// offset 0
		internal ushort 		Magic;					// 	&h10bUS であるべき
		// offset 2
		internal byte			MajorLinkerVersion;	//  &h6US であるべき
		// offset 3
		internal byte			MinorLinkerVersion;	//  &h0US であるべき

		// offset 4
		internal uint			SizeOfCode;
		// offset 8
		internal uint			SizeOfInitializedData;
		// offset 12
		internal uint			SizeOfUninitializedData;
		// offset 16
		internal uint			AddressOfEntryPoint;		// image base 相対
		// offset 20
		internal uint			BaseOfCode;					// image base 相対
		// offset 24
		internal uint			BaseOfData;					// image base 相対

	}		//	struct stStanderdCoffFields

	private ushort[]	STANDERDCOFFFILELDS_MAGIC			= { 0x10b, 		/* PE32  */	
																0x107, 		/* PE32  */
																0x20b		/* PE32+ */
															   };
	private const byte	STANDERDCOFFFILELDS_LMAJOR		= 0x6;
	private const byte	STANDERDCOFFFILELDS_LMINOR		= 0x0;


	internal struct stWindowsNT_specificFields_32
	{
		// offset 0( 28 )
		//メモリに読み込まれる場合のイメージの最初のバイトの優先アドレス。
		//は 64 K の倍数である必要があります。
		//DLL の既定値は 0x10000000 です。
		// Windows CE EXEs の既定値は0x00010000です。
		// Windows NT、Windows 2000、Windows XP、Windows 95、Windows 98、Windows Me の既定値は0x00400000。
		internal uint			ImageBase;					// PE32
		// offset 4( 32 )

		internal uint		SectionAlignment;
		internal uint 		FileAlignment;

		internal ushort		OSMajor;
		internal ushort 	OSMinor;
		internal ushort 	UserMajor;
		internal ushort 	UserMinor;
		internal ushort 	SubSysMajor;
		internal ushort 	SubSysMinor;
		internal uint	 	Reserved;
		internal uint	 	ImageSize;
		internal uint	 	HeaderSize;
		internal uint	 	FileChecksum;
		internal ushort	 	SubSystem;
		internal ushort	 	DLLFlags;
		internal uint	 	StackReserveSize;				// PE32
		internal uint	 	StackCommitSize;				// PE32
		internal uint	 	HeapReserveSize;				// PE32
		internal uint	 	HeapCommitSize;					// PE32
		internal uint	 	LoaderFlags;
		internal uint	 	NumberofDataDirectories;
	}


	internal struct stWindowsNT_specificFields_64
	{
		internal ulong		ImageBase;						// PE32+

		internal uint		SectionAlignment;
		internal uint 		FileAlignment;

		internal ushort		OSMajor;
		internal ushort 	OSMinor;
		internal ushort 	UserMajor;
		internal ushort 	UserMinor;
		internal ushort 	SubSysMajor;
		internal ushort 	SubSysMinor;
		internal uint	 	Reserved;
		internal uint	 	ImageSize;
		internal uint	 	HeaderSize;
		internal uint	 	FileChecksum;
		internal ushort	 	SubSystem;
		internal ushort	 	DLLFlags;
		internal ulong		StackReserveSize;				// PE32+
		internal ulong		StackCommitSize;				// PE32+
		internal ulong		HeapReserveSize;				// PE32+
		internal ulong		HeapCommitSize;					// PE32+
		internal uint	 	LoaderFlags;
		internal uint	 	NumberofDataDirectories;
	}


	private const ushort NTFILEDS_IMAGE_SUBSYSTEM_WINDOWS_CUI	= 0x3;
	private const ushort NTFILEDS_IMAGE_SUBSYSTEM_WINDOWS_GUI	= 0x2;



	internal struct stRVAandSize
	{
		internal uint		VirtualAddress;
		internal uint		Size;
	}


	internal struct stDataDirectories
	{
		// offset 0( 96 )
		internal ulong			ExportTable;
		internal ulong			ImportTable;
		internal ulong			ResourceTable;
		internal ulong			ExceptionTable;
		internal ulong			CertificateTable;
		internal ulong			RelocationTable;
		internal ulong			Debug;
		internal ulong			Copyright;
		internal ulong			GlobalPtr;
		internal ulong			TLSTable;
		internal ulong			LoadConfigTable;
		internal ulong			BoundImport;
		internal ulong			IAT;
		internal ulong			DelayImportDescriptor;
		internal ulong			CLIHeader;
		internal ulong			Reserved;
	}


	//internal unsafe struct stSectionHeader
	internal struct stSectionHeader
	{
		//internal fixed byte 		Name[8];
		internal ulong 				Name;			// ulong 型にしてManaged 構造体で扱えるようにする
		internal uint				VirtualSize;
		internal uint				VirtualAddress;				// image base 相対
		internal uint				SizeOfRawData;
		internal uint				PointerToRawData;
		internal uint				PointerToRelocations;
		internal uint				PointerToLinenumbers;
		internal ushort				NumberOfRelocations;
		internal ushort				NumberOfLinenumbers;
		internal uint				Characteristics;			// enum
	}




	/*****************************************************/
	/*      上位側に返す用マネージド構造体                 */
	/*****************************************************/
	internal class clsPeContentsManaged {
		/* PE file header */
		internal ushort					Machine;
		internal ushort					NumberOfSections;
		internal uint					TimeDateStamp;
		internal uint					PointerToSymbolTable;
		internal uint					 NumberOfSymbols;
		internal ushort					SizeOfOptionalHeader;
		internal ushort					CoffType_Charas;



		internal bool					mblPe32plus;					/* PE32+ */

		/* PE std fields */
		internal ushort 				Magic;
		internal byte					MajorLinkerVersion;
		internal byte					MinorLinkerVersion;

		internal uint					SizeOfCode;
		// offset 8
		internal uint					SizeOfInitializedData;
		// offset 12
		internal uint					SizeOfUninitializedData;
		// offset 16
		internal uint					AddressOfEntryPoint;
		// offset 20
		internal uint					BaseOfCode;
		// offset 24
		internal uint					BaseOfData;

		/* NT-Secific fields */
		internal clsNuint				ImageBase;		// 64bit coff 形式(PE32+)では 64bit化する。
														// なので clsNuint.Size=8 にして保存する。

		internal uint		SectionAlignment;
		internal uint 		FileAlignment;

		internal ushort		OSMajor;
		internal ushort 	OSMinor;
		internal ushort 	UserMajor;
		internal ushort 	UserMinor;
		internal ushort 	SubSysMajor;
		internal ushort 	SubSysMinor;
		internal uint	 	Reserved;
		internal uint	 	ImageSize;
		internal uint	 	HeaderSize;
		internal uint	 	FileChecksum;
		internal ushort	 	SubSystem;
		internal ushort	 	DLLFlags;
		internal clsNuint 	StackReserveSize;				// PE32
		internal clsNuint 	StackCommitSize;				// PE32
		internal clsNuint	HeapReserveSize;				// PE32
		internal clsNuint 	HeapCommitSize;					// PE32
		internal uint	 	LoaderFlags;
		internal uint	 	NumberofDataDirectories;

		//internal stDataDirectories		Directries;
		internal stRVAandSize[]		marOpHhd_Dirs;



		internal clsSectionInf[]		secs;

		internal colonyPECONTENT_FILETYPE FileType {
			get 
			{
				/* ちょっと違うような。 */
				if ( (this.CoffType_Charas & ((ushort)colonyPEFILE_CHARACTERISTICS.IMAGE_FILE_DLL) ) != 0 )
				return colonyPECONTENT_FILETYPE.DLL;
				else
				return colonyPECONTENT_FILETYPE.EXE;
			}
		}

		internal bool IsCoffFormat_Pe32plus
		{
			set { this.mblPe32plus = value;		}
			get { return this.mblPe32plus;		}
		}
	
		public clsPeContentsManaged()
		// コントラスタ
		{
			this.ImageBase = new clsNuint();
		}

	}		// class clsPeContentsManaged

	internal partial class clsSectionInf {
		internal string				Name;
		internal uint				VirtualSize;
		internal uint				VirtualAddress;
		internal uint				SizeOfRawData;
		internal uint				PointerToRawData;
		internal uint				PointerToRelocations;
		internal uint				PointerToLinenumbers;
		internal ushort				NumberOfRelocations;
		internal ushort				NumberOfLinenumbers;
		internal uint				Characteristics;
	}



	/*****************************************************/
	/*      enum 定義                                    */
	/*****************************************************/

	internal enum colonyPEFILE_MACHINE : ushort
	{
		IMAGE_FILE_MACHINE_I386 		= 0x014c
	}

	// https://qiita.com/stc1988/items/e6f76503ddf06fcf20b8
	//			#のenumでビット演算をする
	//				Flags が必要とか怪し過ぎる。。。。
	[System.Flags]
	internal enum colonyPEFILE_CHARACTERISTICS : ushort
	{
		IMAGE_FILE_RELOCS_STRIPPED				= 0x1,			// ECMA335:II.25.2.2.1, Description :0 でないと駄目
		IMAGE_FILE_EXECUTABLE_IMAGE				= 0x2,			// ECMA335:II.25.2.2.1, Description :1 でないと駄目

		IMAGE_FILE_AGGRESSIVE_WS_TRIM 			= 0x10,			// ECMA335 II.25.2.2.1  "implementation-specific"
		IMAGE_FILE_LARGE_ADDRESS_AWARE	 		= 0x20,			// ECMA335 II.25.2.2.1  "implementation-specific"

		IMAGE_FILE_32BIT_MACHINE				= 0x100,		// ECMA335:II.25.2.2.1, Description COMIMAGE_FLAGS_32BITREQUIRED が 1　だったら 1 でないと駄目

		IMAGE_FILE_REMOVABLE_RUN_FROM_SWAP	 	= 0x400,		// ECMA335 II.25.2.2.1  "implementation-specific"
		IMAGE_FILE_NET_RUN_FROM_SWAP			= 0x800,		// ECMA335 II.25.2.2.1  "implementation-specific"

		IMAGE_FILE_DLL							= 0x2000		// ECMA335 II.25.2.2.1 Description  :  DLL 扱いならコレ。
	}


	internal enum colonySUBSYSTEM : ushort
	{
			MAGE_SUBSYSTEM_UNKNOWN 			= 0, 
			MAGE_SUBSYSTEM_NATIVE,
			IMAGE_SUBSYSTEM_WINDOWS_GUI,
			IMAGE_SUBSYSTEM_WINDOWS_CUI, 
			IMAGE_SUBSYSTEM_OS2_CUI , 
			IMAGE_SUBSYSTEM_POSIX_CUI, 
			MAGE_SUBSYSTEM_NATIVE_WINDOWS, 
			IMAGE_SUBSYSTEM_WINDOWS_CE_GUI, 
			IMAGE_SUBSYSTEM_EFI_APPLICATION, 
			IMAGE_SUBSYSTEM_EFI_BOOT_SERVICE_DRIVER , 
			IMAGE_SUBSYSTEM_EFI_RUNTIME_DRIVER, 
			IMAGE_SUBSYSTEM_EFI_ROM, 
			IMAGE_SUBSYSTEM_XBOX, 
			IMAGE_SUBSYSTEM_WINDOWS_BOOT_APPLICATION 
	}

	[System.Flags]
	internal enum colonyDLLFLAGS : ushort
	{
			RESERVED_1			 							= 0x0001, 
			RESERVED_2			 							= 0x0002, 
			
			RESERVED_4							 			= 0x0004, 
			RESERVED_8							 			= 0x0008, 
			IMAGE_DLLCHARACTERISTICS_HIGH_ENTROPY_VA		= 0x0020,
			IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE			= 0x0040, 
			IMAGE_DLLCHARACTERISTICS_FORCE_INTEGRITY		= 0x0080, 
			IMAGE_DLLCHARACTERISTICS_NX_COMPAT				= 0x0100, 
			MAGE_DLLCHARACTERISTICS_NO_ISOLATION 			= 0x0200,
			MAGE_DLLCHARACTERISTICS_NO_SEH 					= 0x0400, 
			MAGE_DLLCHARACTERISTICS_NO_BIND  				= 0x0800, 
			IMAGE_DLLCHARACTERISTICS_APPCONTAINER			= 0x1000, 
			IMAGE_DLLCHARACTERISTICS_DM_DRIVER 				= 0x2000, 
			IMAGE_DLLCHARACTERISTICS_GUARD_CF				= 0x4000, 
			IMAGE_DLLCHARACTERISTICS_TERMINAL_SERVER_AWARE	= 0x8000
	}



	internal enum colonySECTIONHEADER_CHARACTERISTICS : uint
	{
		IMAGE_SCN_CNT_CODE						= 0x00000020,
		IMAGE_SCN_CNT_INITIALIZED_DATA			= 0x00000040,
		IMAGE_SCN_CNT_UNINITIALIZED_DATA		= 0x00000080,	// Section contains uninitialized data. 
		IMAGE_SCN_LNK_OTHER 					= 0x00000100,
		IMAGE_SCN_LNK_INFO						= 0x00000200,
		RESERVED_FOR_FUTURE						= 0x00000400,
		IMAGE_SCN_LNK_COMDAT 					= 0x00001000,
		IMAGE_SCN_GPREL 						= 0x00008000,
		MAGE_SCN_MEM_PURGEABLE					= 0x00020000,
		IMAGE_SCN_MEM_16BIT 					= 0x00020000,
		IMAGE_SCN_MEM_LOCKED 					= 0x00040000, 
		IMAGE_SCN_MEM_PRELOAD					= 0x00080000, 
		IMAGE_SCN_LNK_NRELOC_OVFL				= 0x01000000,
		IMAGE_SCN_MEM_DISCARDABLE				= 0x02000000,
		IMAGE_SCN_MEM_NOT_CACHED				= 0x04000000,
		MAGE_SCN_MEM_NOT_PAGED					= 0x08000000,
		IMAGE_SCN_MEM_SHARED					= 0x10000000,

		IMAGE_SCN_MEM_EXECUTE					= 0x20000000, 	// Section can be executed as code. 
		IMAGE_SCN_MEM_READ						= 0x40000000,	// Section can be read. 
		IMAGE_SCN_MEM_WRITE						= 0x80000000 
	}


	internal enum colonyPECONTENT_FILETYPE
	{
		EXE,
		DLL,
		OBJECT
	}

	// 以下使って無いらしい
	internal enum colonyDIR_TYPE : int
	{
		
		ExportTable = 0,
		ImportTable,
		ResourceTable,
		ExceptionTable,
		CertificateTable,
		RelocationTable,
		Debug,
		Copyright,
		GlobalPtr,
		TLSTable,
		LoadConfigTable,
		BoundImport,
		IAT,
		DelayImportDescriptor,
		CLIHeader,
		Reserved
	}



}		// clsPeFormat




// どうでもいいけど：
// 		https://learn.microsoft.com/ja-jp/dotnet/api/system.reflection.portableexecutable.peheaderbuilder.majorlinkerversion?view=net-7.0
//				PEHeaderBuilder.MajorLinkerVersion プロパティ
//				PE header とか、.Net Core 1.0 以降は出力できるらしい。
//				きったねーー、こっちゃ必死に作ってたのに。


}		// namespace CliDbger

