// csPEformatRead.cs

	// coff 形式( PE 形式) の exe/dll ファイルのrawデータの
	// 入力部分。


	// 当初は VB.net のみで実装予定だったが、VBfixedArray

	// https://learn.microsoft.com/ja-jp/dotnet/api/microsoft.visualbasic.vbfixedarrayattribute?view=net-8.0
	//		VBFixedArrayAttribute クラス
	//			Structure Book
	//				<VBFixedArray(4)> Public Chapter() As Integer
	//			End Structure
	
	// こんな感じで、固定配列をVBでも一応は使えるが、
	// ・中身は VB6 32bit コードではないか？
	// ・将来的に同機能はサポートが打ち切られるのではないのか？
	// ・ちょっと面倒な動きをさせただけで、全く動かないのでは？
	//等々を考慮した結果、
	// ・Csharp の_unsafe struct ... Fixed  宣言を使用。
	// ・上記を CSharpコード上で、マネージド構造体にメンバごと転送。
	//という方式を取る事にした。



using System;






using DWORD = System.UInt32;
using DWORD64 = System.UInt64;

using WORD = System.UInt16;

using BOOL = System.Int32;



namespace CliDbger {



internal partial class clsPeFormat {

	internal System.Collections.Generic.List<colonyRet_CheckPeFormat> marRetWarn;
	private IntPtr						mhFileHandle;
	internal clsPeContentsManaged		mobjPeData;

	/* 以下戻り値テーブルは、
		GetPeformat(), Ret_GetPeformat_GetWinNtFileds()
		で共通にしとく・・・。両方用意するの、けっこうめんどくさいしｗ
	*/
	internal enum colonyRet_CheckPeFormat {
		FAILED_FILEHANDLE_ZERO, 

		FAILED_HEADER_READFILE, 
		FAILED_MSDOSHEADER_UNMATCH,
		FAILED_MSDOSHEADER_SIGNATURE_UNMATCH,

		FAILED_COFFHEADER_MACHINE_ERROR,

		FAILED_HEADER2ND_READFILE,
		FAILED_TO_MAGIC_NUMBER_IN_COFFSTD_FIELDS,


		WARNING_COFFHEADER_POINTER_TOSYMBOL_NOT_ZERO,
		WARNING_COFFHEADER_NUMBEROFSYMBOL_NOT_ZERO,
		WARNING_COFFHEADER_OPTIONAL_HEADER_NOT_MATCH,


		WARNING_STANDERDFIELDS_MAGIC_UNKOWN_NUMBER,
		WARNING_STANDERDFIELDS_LMAJOR_MISMATCH,
		WARNING_STANDERDFIELDS_LMINOR_MISMATCH,


		/* 以下は GetPeformat_GetWinNtFileds_32/64 用 */
		FAILED_NTFIELDS_32_READFILE,
		WARNING_NTFIELDS_32_VERSION_MISMATCH,
		WARNING_NTFIELDS_32_EXESTTING_MISMATCH,
		SUCCCESS_GetPeformat_GetWinNtFileds_32,

		FAILED_NTFIELDS_64_READFILE,
		WARNING_NTFIELDS_64_VERSION_MISMATCH,
		WARNING_NTFIELDS_64_EXESTTING_MISMATCH,
		SUCCCESS_GetPeformat_GetWinNtFileds_64,

		FAILED_DIRECTRIESFIELDS_READFILE,
		SUCCCESS_GetPeformat_GetDataDirectries,

		FAILED_READSECHEDERS_SETFILE_POINTER,
		FAILED_READSECHEDERS_READFILE, 
		SUCCCESS_GetPeformat_GetSectionHeaders,

		SUCCESS_DEBUG_STILL_UNCODED
	}


	internal bool IsWarn () {
		return this.marRetWarn.Count > 0;
	}
	internal System.Collections.Generic.List<colonyRet_CheckPeFormat> GetWarns () {
		return this.marRetWarn;
	}

	internal colonyRet_CheckPeFormat GetPeformat( IntPtr ptrFileHandle )
	{
		this.mhFileHandle = ptrFileHandle;

		if ( this.marRetWarn != null ){
			this.marRetWarn.Clear();
			this.marRetWarn = null;
		}
		if ( this.mobjPeData != null ){
			this.mobjPeData = null;
		}
		if ( this.mhFileHandle == IntPtr.Zero ) {
			return colonyRet_CheckPeFormat.FAILED_FILEHANDLE_ZERO;
		}
		// hFileHandle  :  this.pi.mhFile.DengerousHandle   を想定。プロセスの exe ファイルそのもの


		this.mobjPeData = new clsPeContentsManaged();
		this.marRetWarn = new System.Collections.Generic.List<colonyRet_CheckPeFormat>();

		#if FIRST_FUNC_CHECKPEFORMAT_DEPLOPMENT
			// １．win32 HANDLE を .net System.IO.FileStream に変換しよう。
			// ２．コントラスタから作れるけど FileStream コントラスタは SafeHandle 使わないと駄目なんだね？
			// ３．	https://learn.microsoft.com/ja-jp/dotnet/api/system.runtime.interopservices.safehandle?view=net-8.0
			//				SafeHandle クラス
			//					SafeHandle から派生したクラス
			//					Microsoft.Win32.SafeHandles 名前空間の派生クラスを使用してください。
			//					ファイル ( SafeFileHandle クラス)。

			//					https://learn.microsoft.com/ja-jp/dotnet/api/microsoft.win32.safehandles.safefilehandle?view=net-8.0
			//						SafeFileHandle クラス
			//							「 IDisposable インターフェイス使ってるから注意してね！」
			//							・・・で、何を注意しろ、と？

			//								https://learn.microsoft.com/ja-jp/dotnet/api/system.idisposable?view=net-8.0
			//									IDisposable インターフェイス
			//
			//									要約すると、SafeHandle ラッパークラスを破棄する時に
			//									・using 使わない：.Dispose() メソッド呼んでから破棄してね。
			//									・using 使う　　：自動で Dispose() が呼ばれるから書かないでいいよ。
			//									・・・という事らしい。

			//									・・・で、Finalize() メソッドとデストラクタとどう何が違うわけｗ
			//									Finalize() はいわゆるデストラクタの事で、ガベージコレクタからしか
			//									呼ばれない、らしい。で、ガベージコレクタによる開放はタイミングその他
			//									原則、指定出来ないらしい？
			
			//							if (_handle.IsInvalid)
			//							    throw new Win32Exception(Marshal.GetLastWin32Error(), fileName);
			//							便利そうなコードだ。
			//							Win32API.ReadFile のバッファを byte[] で受け取ってるぞコレ。
			//							どうなってんだ？？？
			//
			//						クラスポインタを初期化しないで直接HANDLE 値を書き込んでる問題。
			//							https://learn.microsoft.com/ja-jp/dotnet/api/system.runtime.interopservices.safehandle?view=net-8.0
			//									SafeHandle クラス

			//									// Open a file, and save its handle in _handle.
			//									// Note that the most optimized code turns into two processor
			//									// instructions: 1) a call, and 2) moving the return value into
			//									// the _handle field.  With SafeHandle, the CLR's platform invoke
			//									// marshaling layer will store the handle into the SafeHandle
			//									// object in an atomic fashion. There is still the problem
			//									// that the SafeHandle object may not be stored in _handle, but
			//									// the real operating system handle value has been safely stored
			//									// in a critical finalizable object, ensuring against leaking
			//									// the handle even if there is an asynchronous exception.

			//									MySafeFileHandle tmpHandle;
			//									tmpHandle = NativeMethods.CreateFile(fileName, NativeMethods.GENERIC_READ,
			//									    FileShare.Read, IntPtr.Zero, FileMode.Open, 0, IntPtr.Zero);

			//									っていう風に直接書き込んでる。
			//									これを実現するために、わざわざ「CriticalFinalizerObject」という
			//									オバケクラスを継承させてる、らしい。どうやら。
			//									SafeHandle 用に専用のモジュールを埋め込んだ、的なオバケコードだけどｗ
			//									技術的には確かに可能だけど・・・将来的に動くのかねこれ？
			//									まいいか、.Net だもんなぁ・・・。
			//									でもランタイムがフリーウェア化してんだし、メンテナンス出来るのかねぇ・・・。
			//									そりゃ _handle メンバーが
			//										MySafeFileHandle tmpHandle;
			//									がさすメモリの先頭に来るのは分かるとしても・・・？？
			//									でもこれ、クラス継承されてメンバー増やされたら駄目なんじゃないの？？？
			//									
			//										MySafeFileHandle tmpHandle;
			//									このエリアに直接２クワットワードとか確保するって事？？
			//									ポインタとポインタがさす領域をごっちゃにしようとか訳分からん事よくやるよなぁ。

			//									超動き怪しいけど・・・まぁいいかｗ
			//									SafeHandle 系は、使ってる人がかなり多い、らしいｗ
			//									強烈にクッソ怪しいんだが・・・。



			// 関係ないけど：
			// https://referencesource.microsoft.com/#mscorlib/system/runtime/interopservices/safehandle.cs
			//			MicroSoft の .net コード宣言一覧？


			// System.IO.FileStream でファイルを開こうと思ったが、
			// こいつの Finalize メソッドのせいで、ファイルハンドルを強制的に
			// 閉じてしまうため、今回は FileStream を使えない。駄目だこりゃｗ

			// IoStream


			//Microsoft.Win32.SafeHandles.SafeFileHandle		sfeFIle;
			//	using ( streamPe = new System.IO.FileStream ) {}
			//try 
			//{
			//	// 万が一考えてハンドルをラップ( 意味ない )
			//	sfeFIle.SetHandle( hFileHandle );

			//}
			//catch( System.Exception e) 
			//{
			//	;
			//}
			//finally {


				//	デストラクタが無理やりファイルを閉じてしまうので、
				//  それを回避する。じゃないと二重に HandleClose してしまう。
				//  これで回避出来ればいいんだけど・・・難しいかなｗ
			//	sfeFIle = null;
			//}
		#endif

		clsMySafeHGlobalHandle			sfeMem;

		DWORD		dwSize;
		dwSize = (DWORD)
			System.Runtime.InteropServices.Marshal.SizeOf( 
					typeof( stPE )
			   );

		using ( sfeMem = new clsMySafeHGlobalHandle(
					System.Runtime.InteropServices.Marshal.AllocHGlobal( 
							(int)dwSize
								) ) )
		{
			{	// nRet, nRead
				int nRet, nReaded;

				nRet = clsWin32.ReadFile( this.mhFileHandle, sfeMem.DangerousGetHandle(),
						(int)dwSize, out nReaded, IntPtr.Zero );

				if ( nRet == 0 || (int)dwSize != nReaded ) {
					// sfeMem わ破棄されるのだろうか・・・。
					return colonyRet_CheckPeFormat.FAILED_HEADER_READFILE;
				}
			}	// nRet, nRead


			unsafe {

				stPE		stPe;

				stPe = (stPE)
					System.Runtime.InteropServices.Marshal.PtrToStructure( 
					sfeMem.DangerousGetHandle(), typeof( stPE )
				);
				
				// PE ヘッダの中身の妥当性をチェックする


				/*********************************************************/
				/* MS DOS header ( 0x80固定 + sognature 0x4バイト)        */
				/*     検査                                               */
				/*********************************************************/
				{		//  nPosDiff
					int nPosDiff;
					for ( nPosDiff = 0; nPosDiff < mbytMSDOS_HeaderChar.Length; nPosDiff++ ) {
						if ( stPe.msdos.hedaderchar[ nPosDiff ] != 
								clsPeFormat.mbytMSDOS_HeaderChar[ nPosDiff ] ) {
							return colonyRet_CheckPeFormat.FAILED_MSDOSHEADER_UNMATCH;
						}
					}

					for ( nPosDiff = 0; nPosDiff < clsPeFormat.mbytMSDOS_PESignature.Length; nPosDiff++ ) {
						if ( stPe.msdos.PESignature[ nPosDiff] != 
								clsPeFormat.mbytMSDOS_PESignature[ nPosDiff ] ) {
							return colonyRet_CheckPeFormat.FAILED_MSDOSHEADER_SIGNATURE_UNMATCH;
						}
					}
				}		// nPosDiff

				/*********************************************************/
				/* Coff ファイルヘッダ( PE file Header)                   */
				/*     検査                                              */
				/*********************************************************/

				if ( stPe.pefile.Machine != (ushort)colonyPEFILE_MACHINE.IMAGE_FILE_MACHINE_I386 ) {
					return colonyRet_CheckPeFormat.FAILED_MSDOSHEADER_SIGNATURE_UNMATCH;
				}
				this.mobjPeData.Machine =  stPe.pefile.Machine;
				this.mobjPeData.NumberOfSections = stPe.pefile.NumberOfSections;

				// TimeDateStamp  ..... みねぇ面倒くせぇｗtime 関数実装めんどくせｗ
				this.mobjPeData.TimeDateStamp = stPe.pefile.TimeDateStamp;

				if ( stPe.pefile.PointerToSymbolTable != 0 )
					this.marRetWarn.Add( colonyRet_CheckPeFormat.WARNING_COFFHEADER_POINTER_TOSYMBOL_NOT_ZERO );
				this.mobjPeData.PointerToSymbolTable = stPe.pefile.PointerToSymbolTable;

				if ( stPe.pefile.NumberOfSymbols != 0 )
					this.marRetWarn.Add( colonyRet_CheckPeFormat.WARNING_COFFHEADER_NUMBEROFSYMBOL_NOT_ZERO );
				this.mobjPeData.NumberOfSymbols = stPe.pefile.NumberOfSymbols;


				{		// 			dwSizeOptionHeader
					DWORD		dwSizeOptionHeader_32, dwSizeOptionHeader_64;
					dwSizeOptionHeader_32 = (DWORD)
								System.Runtime.InteropServices.Marshal.SizeOf( 
											typeof( stOptionalHeader_32 )
										   	);
					dwSizeOptionHeader_64 = (DWORD)
								System.Runtime.InteropServices.Marshal.SizeOf( 
											typeof( stOptionalHeader_64 )
										   	);
					if ( stPe.pefile.SizeOfOptionalHeader != dwSizeOptionHeader_32 && 
						 stPe.pefile.SizeOfOptionalHeader != dwSizeOptionHeader_64 	 )
						this.marRetWarn.Add( colonyRet_CheckPeFormat.WARNING_COFFHEADER_OPTIONAL_HEADER_NOT_MATCH );
					this.mobjPeData.SizeOfOptionalHeader = stPe.pefile.SizeOfOptionalHeader;

				}		// 			dwSizeOptionHeader
				this.mobjPeData.CoffType_Charas = stPe.pefile.Characteristics;


			}		// unsafe


			// sfeMem.Dispose();
		}		// using sfeMem



		dwSize = (DWORD)
			System.Runtime.InteropServices.Marshal.SizeOf( 
					typeof( stStanderdCoffFields )
			   );

		using ( sfeMem = new clsMySafeHGlobalHandle(
					System.Runtime.InteropServices.Marshal.AllocHGlobal( 
							(int)dwSize
								) ) )
		{
			{	// nRet, nRead
				int nRet, nReaded;

				nRet = clsWin32.ReadFile( this.mhFileHandle, sfeMem.DangerousGetHandle(),
						(int)dwSize, out nReaded, IntPtr.Zero );

				if ( nRet == 0 || (int)dwSize != nReaded ) {
					// sfeMem わ破棄されるのだろうか・・・。
					return colonyRet_CheckPeFormat.FAILED_HEADER2ND_READFILE;
				}
			}	// nRet, nRead


			stStanderdCoffFields		stStdCoff;

			stStdCoff = (stStanderdCoffFields)
						System.Runtime.InteropServices.Marshal.PtrToStructure( 
								sfeMem.DangerousGetHandle(), typeof( stStanderdCoffFields )
							);

			/******************************************************************/
			/* 省略可能な(?)ヘッダー標準フィールド( PE header standard fields)  */
			/*     検査                                                       */
			/*****************************************************************/
			{
				//ref  stStanderdCoffFields		stStd;
				//stStd = ref stOpHdr.StanderdCoff;
				int nIndexMagic;
				nIndexMagic = System.Array.IndexOf( this.STANDERDCOFFFILELDS_MAGIC, stStdCoff.Magic );
				if (  nIndexMagic < 0 ) {
					return colonyRet_CheckPeFormat.FAILED_TO_MAGIC_NUMBER_IN_COFFSTD_FIELDS;
				}
				if ( nIndexMagic == 2 )
					this.mobjPeData.IsCoffFormat_Pe32plus = true;
				this.mobjPeData.Magic = stStdCoff.Magic;
				

				if ( stStdCoff.MajorLinkerVersion  != clsPeFormat.STANDERDCOFFFILELDS_LMAJOR ) {
						//Console.WriteLine( "\"{0}\".\"{1}\" Coffheader MajorLinkerVersion  file image:{2}    expect:{3} . ", 
						//	this.ToString(), 
						//	System.Reflection.MethodBase.GetCurrentMethod().Name, 
						//	stStdCoff.MajorLinkerVersion, 
						//	clsPeFormat.STANDERDCOFFFILELDS_LMAJOR );
				
						this.marRetWarn.Add( colonyRet_CheckPeFormat.WARNING_STANDERDFIELDS_LMAJOR_MISMATCH );
				}
				this.mobjPeData.MajorLinkerVersion = stStdCoff.MajorLinkerVersion;

				if ( stStdCoff.MinorLinkerVersion  != clsPeFormat.STANDERDCOFFFILELDS_LMINOR ) 
						this.marRetWarn.Add( colonyRet_CheckPeFormat.WARNING_STANDERDFIELDS_LMINOR_MISMATCH );
				this.mobjPeData.MinorLinkerVersion = stStdCoff.MinorLinkerVersion;
				
				this.mobjPeData.SizeOfCode 				= stStdCoff.SizeOfCode;
				this.mobjPeData.SizeOfInitializedData		= stStdCoff.SizeOfInitializedData;
				this.mobjPeData.SizeOfUninitializedData	= stStdCoff.SizeOfUninitializedData;
				this.mobjPeData.AddressOfEntryPoint		= stStdCoff.AddressOfEntryPoint;
				this.mobjPeData.BaseOfCode				= stStdCoff.BaseOfCode;
				this.mobjPeData.BaseOfData				= stStdCoff.BaseOfData;
			}


			// sfeMem.Dispose();
		}		// using sfeMem



		/* でかい・・・関数分割しよっと。うへ・・・。 */



		colonyRet_CheckPeFormat nRet_GetFields;
		if ( !this.mobjPeData.IsCoffFormat_Pe32plus ) {
			nRet_GetFields = this.GetPeformat_GetWinNtFileds_32();
			if ( nRet_GetFields != colonyRet_CheckPeFormat.SUCCCESS_GetPeformat_GetWinNtFileds_32 ) {
				return nRet_GetFields;
			}


		} else {
			nRet_GetFields = this.GetPeformat_GetWinNtFileds_64();
			if ( nRet_GetFields != colonyRet_CheckPeFormat.SUCCCESS_GetPeformat_GetWinNtFileds_64 ) {
				return nRet_GetFields;
			}
		
		
		}
		

	{
		colonyRet_CheckPeFormat nRet_GetDataDirs;
		nRet_GetDataDirs = this.GetPeformat_GetDataDirectries();
		if ( nRet_GetDataDirs != colonyRet_CheckPeFormat.SUCCCESS_GetPeformat_GetDataDirectries ) {
			return nRet_GetDataDirs;
		}
	}


	{
		colonyRet_CheckPeFormat nRet_GetSectionHeaders;
		nRet_GetSectionHeaders = this.GetPeformat_GetSectionHeaders();
		if ( nRet_GetSectionHeaders != colonyRet_CheckPeFormat.SUCCCESS_GetPeformat_GetSectionHeaders ) {
			return nRet_GetSectionHeaders;
		}
	}


		return colonyRet_CheckPeFormat.SUCCESS_DEBUG_STILL_UNCODED;
		
	}		// colonyRet_CheckPeFormat GetPeformat(...)



	private colonyRet_CheckPeFormat GetPeformat_GetWinNtFileds_32()
	{


		clsMySafeHGlobalHandle		sfeMem;

		DWORD		dwSize;
		dwSize = (DWORD)
			System.Runtime.InteropServices.Marshal.SizeOf( 
					typeof( stWindowsNT_specificFields_32 )
			   );

		using ( sfeMem = new  clsMySafeHGlobalHandle(
							System.Runtime.InteropServices.Marshal.AllocHGlobal( 
							(int)dwSize
								) ) )
		{
			{	// nRet, nRead
				int nRet, nReaded;

				nRet = clsWin32.ReadFile( this.mhFileHandle, sfeMem.DangerousGetHandle(),
						(int)dwSize, out nReaded, IntPtr.Zero );

				if ( nRet == 0 || (int)dwSize != nReaded ) {
					// sfeMem わ破棄されるのだろうか・・・。
					return colonyRet_CheckPeFormat.FAILED_NTFIELDS_32_READFILE;
				}
			}	// nRet, nRead



			stWindowsNT_specificFields_32		stNtFields_32;

			stNtFields_32 = (stWindowsNT_specificFields_32)
				System.Runtime.InteropServices.Marshal.PtrToStructure( 
				sfeMem.DangerousGetHandle(), typeof( stWindowsNT_specificFields_32 )
			);

			/************************************************************************************/
			/* 省略可能な(?)Windows-Specific フィールド( PE header Windows NT-specific fields )  */
			/*     検査                                                                         */
			/***********************************************************************************/

				
			this.mobjPeData.ImageBase.Size = 4;
			this.mobjPeData.ImageBase.Value32Internal = (uint)stNtFields_32.ImageBase;

			this.mobjPeData.SectionAlignment = stNtFields_32.SectionAlignment;
			this.mobjPeData.FileAlignment = stNtFields_32.FileAlignment;


			if ( stNtFields_32.OSMajor  != 5 ||  
				 stNtFields_32.OSMinor  != 0 ||
				 stNtFields_32.UserMajor  != 0 ||
				 stNtFields_32.UserMinor  != 0 ||
				 
				 stNtFields_32.SubSysMajor  != 5 ||
				 stNtFields_32.SubSysMinor  != 0 ) {

				//Console.WriteLine( "\"{0}\".\"{1}\" NT fieldes_32 miss match . ", 
				//	this.ToString(), 
				//	System.Reflection.MethodBase.GetCurrentMethod().Name );
				//Console.WriteLine( "    OSMajor     : file:{0},  expect:{1}", stNtFields_32.OSMajor, 5 );
				//Console.WriteLine( "    OSMinor     : file:{0},  expect:{1}", stNtFields_32.OSMinor, 0 );
				//Console.WriteLine( "    UserMajor   : file:{0},  expect:{1}", stNtFields_32.UserMajor, 0 );
				//Console.WriteLine( "    UserMinor   : file:{0},  expect:{1}", stNtFields_32.UserMinor, 0 );
				//Console.WriteLine( "    SubSysMajor : file:{0},  expect:{1}", stNtFields_32.SubSysMajor, 5 );
				//Console.WriteLine( "    SubSysMinor : file:{0},  expect:{1}", stNtFields_32.SubSysMinor, 0 );

		


				this.marRetWarn.Add( colonyRet_CheckPeFormat.WARNING_NTFIELDS_32_VERSION_MISMATCH );

			}
			this.mobjPeData.OSMajor = stNtFields_32.OSMajor;
			this.mobjPeData.OSMinor = stNtFields_32.OSMinor;
			this.mobjPeData.UserMajor = stNtFields_32.UserMajor;
			this.mobjPeData.UserMinor = stNtFields_32.UserMinor;
			this.mobjPeData.SubSysMajor = stNtFields_32.SubSysMajor;
			this.mobjPeData.SubSysMinor = stNtFields_32.SubSysMinor;


			if ( stNtFields_32.Reserved  != 0 ||  
				 stNtFields_32.FileChecksum  != 0 ||
				( stNtFields_32.SubSystem  != NTFILEDS_IMAGE_SUBSYSTEM_WINDOWS_CUI &&
				  stNtFields_32.SubSystem  != NTFILEDS_IMAGE_SUBSYSTEM_WINDOWS_GUI ) ||
				 ( stNtFields_32.DLLFlags & 0x100f ) != 0 ||
				 stNtFields_32.StackReserveSize != 0x100000 ||
				 stNtFields_32.StackCommitSize  !=  0x1000 ||
				 stNtFields_32.HeapReserveSize  !=  0x100000 ||
				 stNtFields_32.HeapCommitSize  != 0x1000 ||
				 stNtFields_32.LoaderFlags  != 0 ||
				 stNtFields_32.NumberofDataDirectories  != 0x10 ) {

				this.marRetWarn.Add( colonyRet_CheckPeFormat.WARNING_NTFIELDS_32_EXESTTING_MISMATCH );

			}
			this.mobjPeData.Reserved = stNtFields_32.Reserved;
			this.mobjPeData.ImageSize = stNtFields_32.ImageSize;
			this.mobjPeData.HeaderSize = stNtFields_32.HeaderSize;
			this.mobjPeData.FileChecksum = stNtFields_32.FileChecksum;
			this.mobjPeData.SubSystem = stNtFields_32.SubSystem;
			this.mobjPeData.DLLFlags = stNtFields_32.DLLFlags;

			this.mobjPeData.StackReserveSize = new clsNuint( stNtFields_32.StackReserveSize );
			this.mobjPeData.StackCommitSize = new clsNuint( stNtFields_32.StackCommitSize );
			this.mobjPeData.HeapReserveSize = new clsNuint( stNtFields_32.HeapReserveSize );
			this.mobjPeData.HeapCommitSize = new clsNuint( stNtFields_32.HeapCommitSize );


			this.mobjPeData.LoaderFlags = stNtFields_32.LoaderFlags;
			this.mobjPeData.NumberofDataDirectories = stNtFields_32.NumberofDataDirectories;



			// sfeMem.Dispose();
		}		// using sfeMem

		return colonyRet_CheckPeFormat.SUCCCESS_GetPeformat_GetWinNtFileds_32;
	}		//		GetPeformat_GetWinNtFileds_32()







	private colonyRet_CheckPeFormat GetPeformat_GetWinNtFileds_64()
	{


		clsMySafeHGlobalHandle		sfeMem;

		DWORD		dwSize;
		dwSize = (DWORD)
			System.Runtime.InteropServices.Marshal.SizeOf( 
					typeof( stWindowsNT_specificFields_64 )
			   );

		using ( sfeMem = new clsMySafeHGlobalHandle( System.Runtime.InteropServices.Marshal.AllocHGlobal( 
							(int)dwSize
								) ) )
		{
			{	// nRet, nRead
				int nRet, nReaded;

				nRet = clsWin32.ReadFile( this.mhFileHandle, sfeMem.DangerousGetHandle(),
						(int)dwSize, out nReaded, IntPtr.Zero );

				if ( nRet == 0 || (int)dwSize != nReaded ) {
					// sfeMem わ破棄されるのだろうか・・・。
					return colonyRet_CheckPeFormat.FAILED_NTFIELDS_64_READFILE;
				}
			}	// nRet, nRead



			stWindowsNT_specificFields_64		stNtFields_64;

			stNtFields_64 = (stWindowsNT_specificFields_64)
				System.Runtime.InteropServices.Marshal.PtrToStructure( 
				sfeMem.DangerousGetHandle(), typeof( stWindowsNT_specificFields_64 )
			);

			/************************************************************************************/
			/* 省略可能な(?)Windows-Specific フィールド( PE header Windows NT-specific fields )  */
			/*     検査                                                                         */
			/***********************************************************************************/

				
			this.mobjPeData.ImageBase.Size = 8;
			this.mobjPeData.ImageBase.Value64Internal = (ulong)stNtFields_64.ImageBase;

			this.mobjPeData.SectionAlignment = stNtFields_64.SectionAlignment;
			this.mobjPeData.FileAlignment = stNtFields_64.FileAlignment;

			if ( stNtFields_64.OSMajor  != 5 ||  
				 stNtFields_64.OSMinor  != 0 ||
				 stNtFields_64.UserMajor != 0 ||
				 stNtFields_64.UserMinor  != 0 ||
				 
				 ( stNtFields_64.SubSysMajor ) != 5 ||
				 ( stNtFields_64.SubSysMinor ) != 0 ) {

				this.marRetWarn.Add( colonyRet_CheckPeFormat.WARNING_NTFIELDS_64_VERSION_MISMATCH );

				//Console.WriteLine( "\"{0}\".\"{1}\" NT fieldes_64 miss match . ", 
				//	this.ToString(), 
				//	System.Reflection.MethodBase.GetCurrentMethod().Name );
				//Console.WriteLine( "    OSMajor     : file:{0},  expect:{1}", stNtFields_64.OSMajor, 5 );
				//Console.WriteLine( "    OSMinor     : file:{0},  expect:{1}", stNtFields_64.OSMinor, 0 );
				//Console.WriteLine( "    UserMajor   : file:{0},  expect:{1}", stNtFields_64.UserMajor, 0 );
				//Console.WriteLine( "    UserMinor   : file:{0},  expect:{1}", stNtFields_64.UserMinor, 0 );
				//Console.WriteLine( "    SubSysMajor : file:{0},  expect:{1}", stNtFields_64.SubSysMajor, 5 );
				//Console.WriteLine( "    SubSysMinor : file:{0},  expect:{1}", stNtFields_64.SubSysMinor, 0 );

		
			}
			
			this.mobjPeData.OSMajor = stNtFields_64.OSMajor;
			this.mobjPeData.OSMinor = stNtFields_64.OSMinor;
			this.mobjPeData.UserMajor = stNtFields_64.UserMajor;
			this.mobjPeData.UserMinor = stNtFields_64.UserMinor;
			this.mobjPeData.SubSysMajor = stNtFields_64.SubSysMajor;
			this.mobjPeData.SubSysMinor = stNtFields_64.SubSysMinor;

			if ( stNtFields_64.Reserved  != 0 ||  
				 stNtFields_64.FileChecksum  != 0 ||
				( stNtFields_64.SubSystem  != NTFILEDS_IMAGE_SUBSYSTEM_WINDOWS_CUI &&
				  stNtFields_64.SubSystem  != NTFILEDS_IMAGE_SUBSYSTEM_WINDOWS_GUI ) ||
				 ( stNtFields_64.DLLFlags & 0x100f ) != 0 ||
				 stNtFields_64.StackReserveSize != 0x100000 ||
				 stNtFields_64.StackCommitSize  !=  0x1000 ||
				 stNtFields_64.HeapReserveSize  !=  0x100000 ||
				 stNtFields_64.HeapCommitSize  != 0x1000 ||
				 stNtFields_64.LoaderFlags  != 0 ||
				 stNtFields_64.NumberofDataDirectories  != 0x10 ) {

				this.marRetWarn.Add( colonyRet_CheckPeFormat.WARNING_NTFIELDS_64_EXESTTING_MISMATCH );

			}


			this.mobjPeData.Reserved = stNtFields_64.Reserved;
			this.mobjPeData.ImageSize = stNtFields_64.ImageSize;
			this.mobjPeData.HeaderSize = stNtFields_64.HeaderSize;
			this.mobjPeData.FileChecksum = stNtFields_64.FileChecksum;
			this.mobjPeData.SubSystem = stNtFields_64.SubSystem;
			this.mobjPeData.DLLFlags = stNtFields_64.DLLFlags;

			this.mobjPeData.StackReserveSize = new clsNuint( stNtFields_64.StackReserveSize );
			this.mobjPeData.StackCommitSize = new clsNuint( stNtFields_64.StackCommitSize );
			this.mobjPeData.HeapReserveSize = new clsNuint( stNtFields_64.HeapReserveSize );
			this.mobjPeData.HeapCommitSize = new clsNuint( stNtFields_64.HeapCommitSize );


			this.mobjPeData.LoaderFlags = stNtFields_64.LoaderFlags;
			this.mobjPeData.NumberofDataDirectories = stNtFields_64.NumberofDataDirectories;





			// sfeMem.Dispose();
		}		// using sfeMem

		return colonyRet_CheckPeFormat.SUCCCESS_GetPeformat_GetWinNtFileds_64;
	}		//		GetPeformat_GetWinNtFileds_32()


	private colonyRet_CheckPeFormat GetPeformat_GetDataDirectries()
	{
		//windows-win32-debug.pdf
		//		1202 ページ
		//			また、このテーブルの RVA がセクションの先頭を
		//			指していること、または特定のテー
		//			ブルを含むセクションに特定の名前があることを
		//			想定しないでください。
		//
		//			セクションが飛んでいきなり特定名が入るよって事。
		//			あと、あくまでアドレス「テーブル」であって、
		//			いきなりどっかのセクションのど真ん中をさしたりするよって事。


		// Optional header NT fields を呼読んだ時点では
		// セクション情報が無いため場所が確定しない。
		//   .edata セクションなど、アドレス範囲が特定名セクション内にあるかどうかで、
		//   ディレクトリがさす情報を特定させる事にする。

		int		nDirCount;

		nDirCount = (int)this.mobjPeData.NumberofDataDirectories;



		clsMySafeHGlobalHandle		sfeMem;

		DWORD		dwSize;
		dwSize = (DWORD)( nDirCount * 8 );

		using ( sfeMem = new clsMySafeHGlobalHandle( System.Runtime.InteropServices.Marshal.AllocHGlobal( 
							(int)dwSize
								) ) )
		{
			{	// nRet, nRead
				int nRet, nReaded;

				nRet = clsWin32.ReadFile( this.mhFileHandle, sfeMem.DangerousGetHandle(),
						(int)dwSize, out nReaded, IntPtr.Zero );

				if ( nRet == 0 || (int)dwSize != nReaded ) {
					// sfeMem わ破棄されるのだろうか・・・。
					return colonyRet_CheckPeFormat.FAILED_DIRECTRIESFIELDS_READFILE;
				}


			}	// nRet, nRead
			IntPtr		pstDirTop;

			pstDirTop = sfeMem.DangerousGetHandle();
			
			this.mobjPeData.marOpHhd_Dirs = new stRVAandSize[ nDirCount ];
			for ( int nIndex = 0; nIndex < nDirCount; nIndex++ ) {

				IntPtr		pStDir;
				unchecked {
					if (System.IntPtr.Size == 4 ) {
						uint	uiAddr;
						uiAddr = (uint)(pstDirTop.ToInt32()) + (uint)(nIndex * 8 );
						pStDir = new IntPtr( (int)uiAddr );
					} else {
						ulong	ulAddr;
						ulAddr = (ulong)(pstDirTop.ToInt64()) + (ulong)(nIndex * 8 );
						pStDir = new IntPtr( (long)ulAddr );

					}
				}
				this.mobjPeData.marOpHhd_Dirs[ nIndex ] = (stRVAandSize)
						System.Runtime.InteropServices.Marshal.PtrToStructure( 
						pStDir, typeof( stRVAandSize )
					);
			}


		} 	//	using sfeMem
		return colonyRet_CheckPeFormat.SUCCCESS_GetPeformat_GetDataDirectries;

	}		//		colonyRet_CheckPeFormat GetPeformat_GetDataDirectries









	private colonyRet_CheckPeFormat GetPeformat_GetSectionHeaders()
	{
		//windows-win32-debug.pdf
		//		1203 ページ
		//			ファイル ヘッダーで指定されている省略可能な
		//			ヘッダーのサイズを必ず使用してください。
		//
		//			・・・だって。なんか知らんけど使えだと。
		
		int		nSecHeaderNum;
		int		nFilePos;

		nSecHeaderNum = this.mobjPeData.NumberOfSections;

		nFilePos = mbytMSDOS_HeaderChar.Length
					+ mbytMSDOS_PESignature.Length 
					+ (int)System.Runtime.InteropServices.Marshal.SizeOf( 
											typeof( stPeFileHeader )
					 													)
					+ 	(int)this.mobjPeData.SizeOfOptionalHeader;
		
		DWORD	dwNewPos;
		dwNewPos = clsWin32.SetFilePointer( this.mhFileHandle, 
											nFilePos, 
											IntPtr.Zero, 
											(DWORD)clsWin32.colonySETFILEPOINTER_MOVEMETHOD.FILE_BEGIN
											);
		if ( dwNewPos != (DWORD)( nFilePos & 0xffff) ) {
			return colonyRet_CheckPeFormat.FAILED_READSECHEDERS_SETFILE_POINTER;
		}


		clsMySafeHGlobalHandle		sfeMem;

		DWORD		dwSize;
		int			nSecHeaderSizeOf;
		nSecHeaderSizeOf = 
				System.Runtime.InteropServices.Marshal.SizeOf( typeof( stSectionHeader ) );
		dwSize = ((DWORD)nSecHeaderSizeOf) * (DWORD)this.mobjPeData.NumberOfSections;
;

		using ( sfeMem = new clsMySafeHGlobalHandle( System.Runtime.InteropServices.Marshal.AllocHGlobal( 
							(int)dwSize
								) ) )
		{
			{	// nRet, nRead
				int nRet, nReaded;

				nRet = clsWin32.ReadFile( this.mhFileHandle, sfeMem.DangerousGetHandle(),
						(int)dwSize, out nReaded, IntPtr.Zero );

				if ( nRet == 0 || (int)dwSize != nReaded ) {
					// sfeMem わ破棄されるのだろうか・・・。
					return colonyRet_CheckPeFormat.FAILED_READSECHEDERS_READFILE;
				}


			}	// nRet, nRead
			IntPtr		pstDirTop;

			pstDirTop = sfeMem.DangerousGetHandle();
			this.mobjPeData.secs = new clsSectionInf[ nSecHeaderNum ];

			for ( int nIndex = 0; nIndex < nSecHeaderNum; nIndex++ ) {

				IntPtr		pstDir;
				unchecked {
					// ターゲットが 32ビット CPU か 64bit CPU か、なんだよなこれ。
					//  ターゲット=自身 CPU という前提なので・・・。
					if ( System.IntPtr.Size == 4 ) {
						uint	uiAddr;
						uiAddr = (uint)(pstDirTop.ToInt32()) + (uint)(nIndex * nSecHeaderSizeOf );
						pstDir = new IntPtr( (int)uiAddr );
					} else {
						ulong	ulAddr;
						ulAddr = (ulong)(pstDirTop.ToInt64()) + (ulong)(nIndex * nSecHeaderSizeOf );
						pstDir = new IntPtr( (long)ulAddr );

					}
				}

				
				stSectionHeader			stSecHeader;
				clsSectionInf			objSecInf;

				stSecHeader = (stSectionHeader)
						System.Runtime.InteropServices.Marshal.PtrToStructure( 
						pstDir, typeof( stSectionHeader )
					);
				
				objSecInf = new clsSectionInf();
				objSecInf.Name					= clsSectionInf.SetName( stSecHeader.Name );
				objSecInf.VirtualSize			= stSecHeader.VirtualSize;
				objSecInf.VirtualAddress		= stSecHeader.VirtualAddress;
				objSecInf.SizeOfRawData			= stSecHeader.SizeOfRawData;
				objSecInf.PointerToRawData		= stSecHeader.PointerToRawData;


				objSecInf.PointerToRelocations		= stSecHeader.PointerToRelocations;
				objSecInf.PointerToLinenumbers		= stSecHeader.PointerToLinenumbers;
				objSecInf.NumberOfRelocations		= stSecHeader.NumberOfRelocations;
				objSecInf.NumberOfLinenumbers		= stSecHeader.NumberOfLinenumbers;
				

				objSecInf.Characteristics		= stSecHeader.Characteristics;

				this.mobjPeData.secs[ nIndex ] = objSecInf;

			}


		} 	//	using sfeMem

		//		windows-win32-debug.pdf
		//				1246 ページ
		
		//				特別な名前を持つセクション名一覧
		//
		//					とりあえずブレークポイント付近に必要そうなエリアは、
		//					.text ：　実行可能コード(自由形式)
		//					.rdata : 読み取り専用で初期化されたデータ
		//					かな。

		//					


		
		return colonyRet_CheckPeFormat.SUCCCESS_GetPeformat_GetSectionHeaders;

	}		//		colonyRet_CheckPeFormat GetPeformat_GetDataDirectries


	internal partial class clsSectionInf {
		internal static string SetName( ulong ulName ) {

			System.Text.StringBuilder stbRet;

			stbRet = new System.Text.StringBuilder();
			System.Char	ucBuf;
			for ( int nCnt = 0; nCnt < 8; nCnt++ )
			{
				// 00 01 02 03  04 05 06 07
				//   >>> ulong 型で上記を読むと：
				//  big endian : 00-01-02-....
				//  
				//   : ABCD1234
				//     34 12 CD AB 
				ulong		ulBuf;
				ulBuf = ulName >> ( 8 * nCnt );
				ulBuf &= 0xff;
				ucBuf = (char)( ulBuf );

				if ( ucBuf == 0 ) {
					break;
				}
				stbRet.Append( ucBuf );
			}
			return stbRet.ToString();
		}		//		SetName()

	}		// class clsSectionInf

}		// clsPeFormat



//		https://learn.microsoft.com/ja-jp/dotnet/framework/unmanaged-api/hosting/
//				ホスト (アンマネージ API リファレンス)
//			MSCorEE.dll   にアクセスするための関数定義？




}		// namespace CliDbger

