// csSafeHandle.cs

	// SafeHandle ラッパー、DLL/プロセス/スレッド/バッファ用。
	// .net 4.0 以降で実装されてる奴を .net 3.5 でも動くようにしとく。
	//	めんどくせーので一部の Win32API を csWin32.cs を経由せず、ここから
	//	直接呼んでしまっている。リンクの順序守ったりとかめんどくさそうなので。
	//	(直接なんで呼んでるかは理由忘れた)
	// メモリリーク対策にここまでやるとは大変だなｗ



using System;






using DWORD = System.UInt32;
using DWORD64 = System.UInt64;

using WORD = System.UInt16;

using BOOL = System.Int32;


// csharp/vb/win32 vc/vc compiler
//  各種データ型のサイズ詳細メモは csWin32.cs を参照。



namespace CliDbger {




// https://learn.microsoft.com/ja-jp/dotnet/api/system.runtime.interopservices.safehandle?view=net-8.0
//			SafeHandle クラス
//				・・・からコードコピペ。
//				プロセスハンドル、スレッドハンドルでの使用を想定。
//				https://learn.microsoft.com/ja-jp/dotnet/standard/garbage-collection/implementing-dispose
//					Dispose メソッドの実装
//							破棄パターンの実装
//								アンマネージ リソースをラップする SafeHandle から派生したクラス (推奨)、または、
//								Object.Finalize メソッドのオーバーライド。 SafeHandle クラスではファイナライザーが
//								提供されるので、自分で記述する必要はありません。

 
//				なんでデストラクタ相当が無いんだ・・・・。
//				途中で、「finalize 時にやっぱ Api CloseHandle やめて」っていう方法が
//				どうも無いらしい。
//				DEBUG_EVENT 系にはすごく向かないらしい？？？
//				そりゃそーだよなぁ・・・。



	//		https://referencesource.microsoft.com/#mscorlib/microsoft/win32/safehandles/safefilehandle.cs,fc8a018423af3155
	//				Microsoft.Win32.SafeHandles.SafeFileHandle() のソース
	//					まんま使っても問題なさそう。

	//					    [System.Security.SecurityCritical]  // auto-generated_required
	//					    public sealed class SafeFileHandle: SafeHandleZeroOrMinusOneIsInvalid {
	//					 
	//					        private SafeFileHandle() : base(true) 
	//					        {
	//					        }
	//					 
	//					        public SafeFileHandle(IntPtr preexistingHandle, bool ownsHandle) : base(ownsHandle) {
	//					            SetHandle(preexistingHandle);
	//					        }
	//					 
	//					        [System.Security.SecurityCritical]
	//					        [ResourceExposure(ResourceScope.Machine)]
	//					        [ResourceConsumption(ResourceScope.Machine)]
	//					        override protected bool ReleaseHandle()
	//					        {
	//					            return Win32Native.CloseHandle(handle);
	//					        }
	//					    }

internal class clsMySafeCloseHandle : Microsoft.Win32.SafeHandles.SafeHandleZeroOrMinusOneIsInvalid
{
	// Create a SafeHandle, informing the base class
	// that this SafeHandle instance "owns" the handle,
	// and therefore SafeHandle should call
	// our ReleaseHandle method when the SafeHandle
	// is no longer in use.
	private clsMySafeCloseHandle()
		: base(true)
	{
	}

	internal clsMySafeCloseHandle(IntPtr handle)
			: base(true)
	{
		//iagnosticUtility.DebugAssert(handle == IntPtr.Zero || !ownsHandle, "Unsafe to create a SafeHandle that owns a pre-existing handle before the SafeHandle was created.");
		base.SetHandle(handle);
	}

	[System.Runtime.ConstrainedExecution.ReliabilityContract(System.Runtime.ConstrainedExecution.Consistency.WillNotCorruptState, System.Runtime.ConstrainedExecution.Cer.MayFail)]
	override protected bool ReleaseHandle()
	{
		// Here, we must obey all rules for constrained execution regions.
		return clsWin32ForSafeHandle.CloseHandle(handle);
		// If ReleaseHandle failed, it can be reported via the
		// "releaseHandleFailed" managed debugging assistant (MDA).  This
		// MDA is disabled by default, but can be enabled in a debugger
		// or during testing to diagnose handle corruption problems.
		// We do not throw an exception because most code could not recover
		// from the problem.
	}

	
	// https://referencesource.microsoft.com/#mscorlib/system/runtime/interopservices/safehandle.cs,d50ea173b91f239b,references
	//		IsInvalid ソース？
	//				を真似する。

	override public bool IsInvalid {
	[System.Runtime.ConstrainedExecution.ReliabilityContract(System.Runtime.ConstrainedExecution.Consistency.WillNotCorruptState, System.Runtime.ConstrainedExecution.Cer.Success)]
		get {
			if ( this.handle == IntPtr.Zero || this.handle.ToInt64() == -1 )
				return false;
			return true;
		}
	}

}		// class clsMySafeCloseHandle




//				DLLハンドル開放用( 上のコピペ )
internal class clsMySafeFreeLibraryHandle : Microsoft.Win32.SafeHandles.SafeHandleZeroOrMinusOneIsInvalid
{

	private clsMySafeFreeLibraryHandle()
		: base(true)
	{
	}
	internal clsMySafeFreeLibraryHandle(IntPtr handle)
			: base(true)
	{
		//DiagnosticUtility.DebugAssert(handle == IntPtr.Zero || !ownsHandle, "Unsafe to create a SafeHandle that owns a pre-existing handle before the SafeHandle was created.");
		if ( handle == IntPtr.Zero )
			throw new System.Exception( "handle value is size zero.");

		base.SetHandle(handle);
	}


	[System.Runtime.ConstrainedExecution.ReliabilityContract(System.Runtime.ConstrainedExecution.Consistency.WillNotCorruptState, System.Runtime.ConstrainedExecution.Cer.MayFail)]
	override protected bool ReleaseHandle()
	{
		return clsWin32ForSafeHandle.FreeLibrary(handle);
	}


	override public bool IsInvalid {
	[System.Runtime.ConstrainedExecution.ReliabilityContract(System.Runtime.ConstrainedExecution.Consistency.WillNotCorruptState, System.Runtime.ConstrainedExecution.Cer.Success)]
		get {
			if ( this.handle == IntPtr.Zero || this.handle.ToInt64() == -1 )
				return false;
			return true;
		}
	}

}		// class clsMySafeFreeLibraryHandle



//	FreeHGlobal(プロセス heap メモリ) ハンドル開放用( 上のコピペ )

	//		https://referencesource.microsoft.com/#System.IdentityModel/System/IdentityModel/SspiSafeHandles.cs,d2a4a3ce370fd3d4
	//			MS clsMySafeHGlobalHandle ソース
	//				・・・なんだけど、System.IdentityModel とかいう訳の分からん所のなんだよな。
	//				つまり・・・「どこぞ訳の分からん所で実装された関数」って意味なのかな？
	//				自力で実装しないと駄目らしい。
	//				
	//				    [SecurityCritical(SecurityCriticalScope.Everything)]
	//				#pragma warning restore 618
	//				    sealed class clsMySafeHGlobalHandle : SafeHandleZeroOrMinusOneIsInvalid
	//				    {
	//				        clsMySafeHGlobalHandle() : base(true) { }
	//				 
	//				        // 0 is an Invalid Handle
	//				        clsMySafeHGlobalHandle(IntPtr handle)
	//				            : base(true)
	//				        {
	//				            DiagnosticUtility.DebugAssert(handle == IntPtr.Zero, "clsMySafeHGlobalHandle constructor can only be called with IntPtr.Zero.");
	//				            SetHandle(handle);
	//				        }
	//				 
	//				        protected override bool ReleaseHandle()
	//				        {
	//				            Marshal.FreeHGlobal(handle);
	//				            return true;
	//				        }
	//				 
	//				        public static clsMySafeHGlobalHandle InvalidHandle
	//				        {
	//				            get { return new clsMySafeHGlobalHandle(IntPtr.Zero); }
	//				        }
	//				 
	//				        public static clsMySafeHGlobalHandle AllocHGlobal(string s)
	//				        {
	//				            byte[] bytes = DiagnosticUtility.Utility.AllocateByteArray(checked((s.Length + 1) * 2));
	//				            Encoding.Unicode.GetBytes(s, 0, s.Length, bytes, 0);
	//				            return AllocHGlobal(bytes);
	//				        }
	//				 
	//				        public static clsMySafeHGlobalHandle AllocHGlobal(byte[] bytes)
	//				        {
	//				            clsMySafeHGlobalHandle result = AllocHGlobal(bytes.Length);
	//				            Marshal.Copy(bytes, 0, result.DangerousGetHandle(), bytes.Length);
	//				            return result;
	//				        }
	//				 
	//				        public static clsMySafeHGlobalHandle AllocHGlobal(uint cb)
	//				        {
	//				            // The cast could overflow to minus.
	//				            // Unfortunately, Marshal.AllocHGlobal only takes int.
	//				            return AllocHGlobal((int)cb);
	//				        }
	//				 
	//				        public static clsMySafeHGlobalHandle AllocHGlobal(int cb)
	//				        {
	//				            if (cb < 0)
	//				            {
	//				                throw DiagnosticUtility.ExceptionUtility.ThrowHelperError(new ArgumentOutOfRangeException("cb", SR.GetString(SR.ValueMustBeNonNegative)));
	//				            }
	//				 
	//				            clsMySafeHGlobalHandle result = new clsMySafeHGlobalHandle();
	//				 
	//				            // CER 
	//				            RuntimeHelpers.PrepareConstrainedRegions();
	//				            try { }
	//				            finally
	//				            {
	//				                IntPtr ptr = Marshal.AllocHGlobal(cb);
	//				                result.SetHandle(ptr);
	//				            }
	//				            return result;
	//				        }
	//				    }

	//				・・・のほとんどを丸まるコピペ。訳分からんトコは削る。
	//				.net 3.5 には実装されてないので、あれば助かる・・・かな？

internal class clsMySafeHGlobalHandle : Microsoft.Win32.SafeHandles.SafeHandleZeroOrMinusOneIsInvalid
    {
        internal clsMySafeHGlobalHandle() : base(true) { }
 
        // 0 is an Invalid Handle
        internal clsMySafeHGlobalHandle(IntPtr handle)
            : base(true)
        {
            //DiagnosticUtility.DebugAssert(handle == IntPtr.Zero, "clsMySafeHGlobalHandle constructor can only be called with IntPtr.Zero.");
			if ( handle == IntPtr.Zero ||  handle == new IntPtr(-1) )
				throw new System.Exception( "handle value is size zero.");

            SetHandle(handle);
        }
 
        protected override bool ReleaseHandle()
        {
            System.Runtime.InteropServices.Marshal.FreeHGlobal(handle);
            return true;
        }
 
        public static clsMySafeHGlobalHandle InvalidHandle
        {
            get { return new clsMySafeHGlobalHandle(IntPtr.Zero); }
        }
 
        public static clsMySafeHGlobalHandle AllocHGlobal(string s)
        {
            //byte[] bytes = DiagnosticUtility.Utility.AllocateByteArray(checked((s.Length + 1) * 2));
			byte[] bytes = new byte [ (checked((s.Length + 1) * 2)) ];
            System.Text.Encoding.Unicode.GetBytes(s, 0, s.Length, bytes, 0);
            return AllocHGlobal(bytes);
        }
 
        public static clsMySafeHGlobalHandle AllocHGlobal(byte[] bytes)
        {
            clsMySafeHGlobalHandle result = AllocHGlobal(bytes.Length);
            System.Runtime.InteropServices.Marshal.Copy(bytes, 0, result.DangerousGetHandle(), bytes.Length);
            return result;
        }
 
        public static clsMySafeHGlobalHandle AllocHGlobal(uint cb)
        {
            // The cast could overflow to minus.
            // Unfortunately, Marshal.AllocHGlobal only takes int.
            return AllocHGlobal((int)cb);
        }
 
        public static clsMySafeHGlobalHandle AllocHGlobal(int cb)
        {
            if (cb < 0)
            {
                //throw DiagnosticUtility.ExceptionUtility.ThrowHelperError(new ArgumentOutOfRangeException("cb", SR.GetString(SR.ValueMustBeNonNegative)));
				throw new System.ArgumentOutOfRangeException("cb", "-----");
            }
 
            clsMySafeHGlobalHandle result = new clsMySafeHGlobalHandle();
 
            // CER 
            System.Runtime.CompilerServices.RuntimeHelpers.PrepareConstrainedRegions();
            try { }
            finally
            {
                IntPtr ptr = System.Runtime.InteropServices.Marshal.AllocHGlobal(cb);
                result.SetHandle(ptr);
            }
            return result;
        }
    }




[System.Security.SuppressUnmanagedCodeSecurity()]
internal class clsWin32ForSafeHandle
{


	// Free the kernel's file object (close the file).
	[System.Runtime.InteropServices.DllImport("kernel32", SetLastError = true)]
	[System.Runtime.ConstrainedExecution.ReliabilityContract(System.Runtime.ConstrainedExecution.Consistency.WillNotCorruptState, System.Runtime.ConstrainedExecution.Cer.MayFail)]
	internal extern static bool CloseHandle(IntPtr handle);

	[System.Runtime.InteropServices.DllImport("kernel32.dll", SetLastError = true)]
	[System.Runtime.ConstrainedExecution.ReliabilityContract(System.Runtime.ConstrainedExecution.Consistency.WillNotCorruptState, System.Runtime.ConstrainedExecution.Cer.MayFail)]
	internal extern static bool FreeLibrary(
						IntPtr hLibModule
				);

}		// class clsWin32ForSafeHandle


#if RESEACH_SAFEHANDLE
	// SafeHandle:
	//				ガベージコレクタやら、ローカル変数の自動 Dispose やら、とにかく
	//				メモリリークやハンドル重複現象が色々起こりまくるらしい。

	//				根本的には
	//					・いらなくなったら dispose を丁寧に呼ぶ
	//					・uging 使う
	//					・safehandle から safehandle のコピーは作らない
	//				などなど、かなり神経質にならんと駄目になってしまっている。


#endif



}		// namespace CliDbger

